package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;

public class QuoteMassUpdateAction<ATTACHEDTO extends Help> extends Action {

    private String qmuLabel;
    private Boolean calculate;
    private Boolean copy;
    private Boolean simulation = true;
    private String copyLabel;
    private String copyComment;
    private ATTACHEDTO backLink;

    public QuoteMassUpdateAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations, String qmuLabel) {
        super(invocations);
        this.backLink = backLink;
        this.qmuLabel = qmuLabel;
        this.calculate = false;
        this.copy = false;
    }

    /**
     * Whether the action is valid and should be triggered or not.
     *
     * @param calculate <ul>
     *                             <li>true - calculation of the QMU is started as background
     *                             process</li>
     *                             <li>false - calculation of the QMU is NOT started at all</li>
     *                             </ul>
     */
    public QuoteMassUpdateAction<ATTACHEDTO> setCalculate(Boolean calculate) {
        invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCalculate", calculate));
        this.calculate = Boolean.TRUE.equals(calculate) ? true : false;
        return this;
    }

    /**
     * Whether the action should copy a qoute mass update object and the run the copied one or not
     *
     * @param copy <ul>
     *                             <li>true - a quote mass update should be copied before execution</li>
     *                             <li>false - default, the execution will run on the original object</li>
     *                             </ul>
     */
    public QuoteMassUpdateAction<ATTACHEDTO> setCopy(Boolean copy) {
        invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCopy", copy));
        this.copy = Boolean.TRUE.equals(copy) ? true : false;
        return this;
    }

    /**
     * Whether the action on copied object should be a simulation or not
     *
     * @param simulation true by default
     */
    public QuoteMassUpdateAction<ATTACHEDTO> setSimulation(Boolean simulation) {
        invocations.add(MethodInvocationRecord.of(getObjectName() + ".setSimulation", simulation));
        this.simulation = Boolean.TRUE.equals(simulation) ? true : false;
        return this;
    }

    /**
     * If copy is true, what should be the new label of the copied qoute mass update
     *
     * @param label
     */
    public QuoteMassUpdateAction<ATTACHEDTO> setCopyLabel(String label) {
        invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCopyLabel", label));
        this.copyLabel = label;
        return this;
    }

    /**
     * Comment that is passed also into submitter reason into workflow
     *
     * @param copyComment
     */
    public QuoteMassUpdateAction<ATTACHEDTO> setCopyComment(String copyComment) {
        invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCopyComment", copyComment));
        this.copyComment = copyComment;
        return this;
    }

    public ATTACHEDTO back() {
        invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
        return backLink;
    }

    @Override
    public String getObjectName() {
        return backLink.getObjectName() + ".QuoteMassUpdateAction";
    }


    public String getQuoteMassUpdateLabel() {
        return qmuLabel;
    }

    public Boolean getCalculate() {
        return calculate;
    }

    public Boolean getCopy() {
        return calculate;
    }

    public Boolean getSimulation() {
        return simulation;
    }

    public String getCopyLabel() {
        return copyLabel;
    }

    public String getCopyComment() {
        return copyComment;
    }

    @Override
    public String toString() {
        return "QuoteMassUpdateAction [label=" + qmuLabel + ", calculate=" + calculate + ", copy=" + copy + ", copyLabel=" + copyLabel + ", comment=" + copyComment + ", backLink=" + backLink + ", hashCode()=" + hashCode() + "]";
    }

}