package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class ScatterChart extends AbstractBuilder {

	public ScatterChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	protected ScatterOptions options;
	protected Dictionary<ScatterChart> dictionary;
	
	public ScatterSeries addSeries() {
		String id = addInvocation("addSeries");
		return new ScatterSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public ScatterOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new ScatterOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<ScatterChart> addAuxLine() {
		String id = addInvocation("addAuxLine");
		return new AuxLine<ScatterChart>(this, id, "AuxLine");
	}
	
	public Dictionary<ScatterChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.SCATTER;
		return r;
	}

	
	public static class ScatterOptions extends AbstractOptions<ScatterChart,ScatterOptions> {

		public ScatterOptions(ScatterChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public ScatterOptions setXMin(BigDecimal xMin) {
			
			addInvocation("setXMin",xMin);
			return this;
		}
		
		public ScatterOptions setXMax(BigDecimal xMax) {
			
			addInvocation("setXMax",xMax);
			return this;
		}
				
		public ScatterOptions setMarker(BigDecimal xValue, BigDecimal yValue) {

			addInvocation("setMarker",xValue, yValue);
			return this;
		}
		
		public ScatterOptions setDisableDrilldown(Boolean disableDrilldown) {

			addInvocation("setDisableDrilldown", disableDrilldown);
			return this;
		}
		
		public ScatterOptions setDisableZoom(Boolean disableZoom) {

			addInvocation("setDisableZoom", disableZoom);
			return this;
		}
		
	}
	
	public static class ScatterSeries extends AbstractSeries<ScatterChart,ScatterSeries> {

		public ScatterSeries(ScatterChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public AggrFieldAxis<ScatterSeries> setAxisX(String axisX) {

			String id = addInvocation("setAxisX",axisX);
			return new AggrFieldAxis<ScatterSeries>(this, id, "AxisX");
		}
		
		public AggrFieldAxis<ScatterSeries> setAxisY(String axisY) {

			String id = addInvocation("setAxisY",axisY);
			return new AggrFieldAxis<ScatterSeries>(this, id, "AxisY");
		}

		public ScatterSeries setAggregationBy(String aggregationBy) {

			addInvocation("setAggregationBy",aggregationBy);
			return this;
		}
		
		public ScatterSeries setBandBy(String bandBy) {

			addInvocation("setBandBy",bandBy);
			return this;
		}
		
		public ScatterSeries setShowBandByLegend(Boolean showBandByLegend) {

			addInvocation("setShowBandByLegend",showBandByLegend);
			return this;
		}
		
		public ScatterAdditionalMeasure<ScatterSeries> addAdditionalMeasure() {
			
			String id = addInvocation("addAdditionalMeasure");
			return new ScatterAdditionalMeasure<ScatterSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public TrendLine<ScatterSeries> withTrendLine() {
			
			String id = addInvocation("withTrendLine");
			return new TrendLine<ScatterSeries>(this, id, "TrendLine");
		}
		
		public Formula<ScatterSeries> setFormula() {
			
			String id = addInvocation("setFormula");
			return new Formula<ScatterSeries>(this, id, "Formula");
		}
		
		public ScatterSeries setGeneratedQueryDto(String generatedQueryDto) {

			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}

	}

	public static class ScatterAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,ScatterAdditionalMeasure<ATTACHEDTO>> {
		 
		public ScatterAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}
		
		public Expression<ScatterAdditionalMeasure<ATTACHEDTO>> withExpression() { 

			String id = addInvocation("withExpression");
			return new Expression<ScatterAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}		
	}
}
