package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;
import java.util.Date;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class TimeSeriesScatterChart extends AbstractBuilder {

	public TimeSeriesScatterChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}
	
	protected TimeSeriesScatterOptions options;
	protected Dictionary<TimeSeriesScatterChart> dictionary;
	
	public TimeSeriesScatterSeries addSeries() {
		String id = addInvocation("addSeries");
		return new TimeSeriesScatterSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public TimeSeriesScatterOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new TimeSeriesScatterOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<TimeSeriesScatterChart> addAuxLine() {
		String id = addInvocation("addAuxLine");
		return new AuxLine<TimeSeriesScatterChart>(this, id, "AuxLine");
	}
	
	public Dictionary<TimeSeriesScatterChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.TIMESERIES_SCATTER;
		return r;
	}

	
	public static class TimeSeriesScatterOptions extends AbstractOptions<TimeSeriesScatterChart,TimeSeriesScatterOptions> {

		public TimeSeriesScatterOptions(TimeSeriesScatterChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public TimeSeriesScatterOptions setXMin(Date xMin) {

			addInvocation("setXMin",xMin);
			return this;
		}
		
		public TimeSeriesScatterOptions setXMax(Date xMax) {

			addInvocation("setXMax",xMax);
			return this;
		}
		
		
		public TimeSeriesScatterOptions setMarker(BigDecimal xValue, BigDecimal yValue) {

			addInvocation("setMarker",xValue, yValue);
			return this;
		}
	}
	
	public static class TimeSeriesScatterSeries extends AbstractSeries<TimeSeriesScatterChart,TimeSeriesScatterSeries> {

		public TimeSeriesScatterSeries(TimeSeriesScatterChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public AggrFieldAxis<TimeSeriesScatterSeries> setAxisX(String axisX) {
			String id = addInvocation("setAxisX",axisX);
			return new AggrFieldAxis<TimeSeriesScatterSeries>(this, id, "AxisX");
		}
		
		public AggrFieldAxis<TimeSeriesScatterSeries> setAxisY(String axisY) {
			String id = addInvocation("setAxisY",axisY);
			return new AggrFieldAxis<TimeSeriesScatterSeries>(this, id, "AxisY");
		}

		public TimeSeriesScatterSeries setAggregationBy(String aggregationBy) {
			
			addInvocation("setAggregationBy",aggregationBy);
			return this;
		}
		
		public TimeSeriesScatterSeries setBandBy(String bandBy) {
			
			addInvocation("setBandBy",bandBy);
			return this;
		}
		
		public TimeSeriesScatterSeries setShowBandByLegend(Boolean showBandByLegend) {
			
			addInvocation("setShowBandByLegend",showBandByLegend);
			return this;
		}
		
		public TimeSeriesScatterAdditionalMeasure<TimeSeriesScatterSeries> addAdditionalMeasure() {
			String id = addInvocation("addAdditionalMeasure");
			return new TimeSeriesScatterAdditionalMeasure<TimeSeriesScatterSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<TimeSeriesScatterSeries> setFormula() {
			String id = addInvocation("setFormula");
			return new Formula<TimeSeriesScatterSeries>(this, id, "Formula");
		}

		public TimeSeriesScatterSeries setGeneratedQueryDto(String generatedQueryDto) {
			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}		

	}

	public static class TimeSeriesScatterAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,TimeSeriesScatterAdditionalMeasure<ATTACHEDTO>> {
		 
		public TimeSeriesScatterAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}

		
		public Expression<TimeSeriesScatterAdditionalMeasure<ATTACHEDTO>> withExpression() { 
			String id = addInvocation("withExpression");
			return new Expression<TimeSeriesScatterAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}
		
	}
}
