/*
 *
 */

package net.pricefx.formulaengine;

/**
 *
 * This is the base class for all exceptions that can occurs using the
 *
 * Expression Class.
 *
 * @see net.pricefx.formulaengine.Expression
 *
 * @see net.pricefx.formulaengine.XIllegalOperation
 *
 * @see net.pricefx.formulaengine.XIllegalStatus
 *
 * @see net.pricefx.formulaengine.XUndefinedVariable
 *
 * @see net.pricefx.formulaengine.XUndefinedFunction
 *
 * @see net.pricefx.formulaengine.XUndefinedUnit
 */

public class XExpression extends Exception {

    /**
     *
     */
    private static final long serialVersionUID = 8963214792215339989L;
    private int startPosition;
    private int endPosition;
    private String sourceElement;

    public XExpression(String errorMessage) {
        super(errorMessage);
        this.startPosition = 0;
        this.endPosition = 0;
    }

    public XExpression(Throwable cause) {
        super(cause.getMessage(), cause);
        this.startPosition = 0;
        this.endPosition = 0;
    }

    public XExpression(int startPosition, Throwable cause) {
        this(startPosition, startPosition+1, cause);
    }

    public XExpression(int startPosition, int endPosition, Throwable cause) {

        super("ERROR(@" + startPosition + "): " + cause.getMessage(), cause);
        this.startPosition = startPosition;
        this.endPosition = endPosition;
    }

    /**
     *
     * Gets the start position where the error occurred.
     *
     * @return the position of the problem.
     */
    public int getStartPosition() {
        return startPosition;
    }

    /**
     * Gets the end position where the error occurred.
     *
     * @return
     */
    public int getEndPosition() {
        return endPosition;
    }

    public String getSourceElement() {
        return sourceElement;
    }

    public void setSourceElement(String sourceElement) {
        this.sourceElement = sourceElement;
    }
}
